package no.ntnu.item.arctis.examples.android.cityguide.getperiodiclocationupdates;


import android.app.Service;
import android.content.Intent;
import android.location.Geocoder;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.os.IBinder;
import android.util.Log;

public class LocationService extends Service implements LocationListener {

    private LocationManager locationManager;
    private Geocoder geocoder;
    
    private static LocationService instance;
    
    private boolean gps_enabled = false, network_enabled= false;
	
    String TAG = "LocationService";
    
    String gpsProvider = LocationManager.GPS_PROVIDER;
    String networkProvider = LocationManager.NETWORK_PROVIDER;

    
	@Override
	public void onCreate() {
		
	    locationManager = (LocationManager)this.getSystemService(LOCATION_SERVICE);
		this.locationManager.removeUpdates(this);
	    geocoder = new Geocoder(this); 
    
	    //**/ uncomment the line below to make use of efficient location provider
	   // updateLocationProvider();
	    //*/
	  	
        this.locationManager.requestLocationUpdates(networkProvider, 60000, 0,	this);  // updates in every 30 sec		
	    
        // Uncomment the following block to get the last known location       
	    Location location = locationManager.getLastKnownLocation(networkProvider); 
	    if (location != null) {
	      Log.d(TAG, "onCreate_location "+location.toString());
	      this.onLocationChanged(location); 
	    }
	    else{
	    	if(GetPeriodicLocationUpdates.getInstance() !=null){
	    		GetPeriodicLocationUpdates.getInstance().sendToThisBlock("LOCATION_NOT_FOUND");
	    	}
	    }
	    //
	    
	    instance = this;
		
		super.onCreate();
	}
	
	private void updateLocationProvider(){
	    //**/
	    //check if providers are enabled
        try{gps_enabled=locationManager.isProviderEnabled(gpsProvider);}catch(Exception ex){}
        try{network_enabled=locationManager.isProviderEnabled(networkProvider);}catch(Exception ex){}
        
        if(gps_enabled && network_enabled){
        	//if both enabled, choose network
        	this.locationManager.requestLocationUpdates(gpsProvider, 15000, 0,	this);
        	System.out.println("Both are enabled and gps is chosen");
        } else if(network_enabled && !gps_enabled){
        	// only network is enabled
        	this.locationManager.requestLocationUpdates(networkProvider, 15000, 0,	this);
        	System.out.println("only network_enabled and is chosen");
        } else if(gps_enabled){
        	// only gps is enabled
        	this.locationManager.requestLocationUpdates(gpsProvider, 15000, 0,	this);
        	System.out.println("gps_enabled and is chosen");
        } else if(!gps_enabled && !network_enabled) {
        	// both are disabled
        	System.out.println(" both are disabled");
        	if(GetPeriodicLocationUpdates.getInstance() !=null){
	    		GetPeriodicLocationUpdates.getInstance().sendToThisBlock("LOCATION_NOT_FOUND");
	    	}
        }
	    //*/
	}
	
	public static LocationService getInstance(){
		return instance;
	}

	@Override
	public void onDestroy() {
		// TODO Auto-generated method stub
		Log.d(TAG, TAG + ":onDestroy");
		super.onDestroy();
	}

	@Override
	public void onStart(Intent intent, int startId) {
		// TODO Auto-generated method stub
		super.onStart(intent, startId);
	}

	@Override
	public void onLocationChanged(Location location) {
		// TODO Auto-generated method stub
	    Log.d(TAG, "onLocationChanged with location " + location.toString());
	    String text = String.format("Lat:\t %f\nLong:\t %f\nAlt:\t %f\nBearing:\t %f", location.getLatitude(), 
	                  location.getLongitude(), location.getAltitude(), location.getBearing());
	    System.out.println("onLocationChanged sssss location "+ text);
	    
	    if(GetPeriodicLocationUpdates.getInstance() !=null){
	    	GetPeriodicLocationUpdates.getInstance().sendToThisBlock("LOCATION_CHANGED", location);
	    }		 
	  
	}

	@Override
	public void onProviderDisabled(String provider) {
		// TODO Auto-generated method stub
		Log.d(TAG, TAG + ":providerDisabled");
		
		//updateLocationProvider();
	}

	@Override
	public void onProviderEnabled(String provider) {
		// TODO Auto-generated method stub		
		Log.d(TAG, TAG + ":onProviderEnabled");
		
		//updateLocationProvider();	
	}

	@Override
	public void onStatusChanged(String provider, int status, Bundle extras) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public IBinder onBind(Intent intent) {
		// TODO Auto-generated method stub
		return null;
	}
}
