package no.ntnu.item.arctis.examples.android.cityguide.getperiodiclocationupdates;

import java.util.Timer;
import java.util.TimerTask;

import android.content.Context;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;

public class MyLocation {
	private Timer timer;
	private LocationManager lm;
	private LocationResult locationResult;
	private boolean gps_enabled = false;
	private boolean network_enabled = false;
	private long delay;
	
	public MyLocation(long delay) {
		this.delay = delay;
		timer = new Timer();
	}
	
	public boolean anyProviderEnabled(Context context) {
		if (lm == null) {
			lm = (LocationManager) context.getSystemService(Context.LOCATION_SERVICE);
		}

		gps_enabled = lm.isProviderEnabled(LocationManager.GPS_PROVIDER);
		network_enabled = lm.isProviderEnabled(LocationManager.NETWORK_PROVIDER);

		//don't start listeners if no provider is enabled
		if(!gps_enabled && !network_enabled) {
			return false;
		}
		return true;
	}

	public void getLocation(LocationResult result) {
		//LocationResult callback class is used to pass location value from MyLocation to user code.
		locationResult = result;

		if (gps_enabled) {
			lm.requestLocationUpdates(LocationManager.GPS_PROVIDER, 0, 0, locationListenerGps);
		}
		if (network_enabled) {
			lm.requestLocationUpdates(LocationManager.NETWORK_PROVIDER, 0, 0, locationListenerNetwork);
		}
		timer.schedule(new GetLastLocation(), delay);
	}
	
	public void removeListeners() {
		if (locationListenerGps != null) {
			lm.removeUpdates(locationListenerGps);
		}
		if (locationListenerNetwork != null) {
			lm.removeUpdates(locationListenerNetwork);
		}
	}

	LocationListener locationListenerGps = new LocationListener() {
		public void onLocationChanged(Location location) {
			timer.cancel();
			locationResult.gotLocation(location);
			lm.removeUpdates(this);
			lm.removeUpdates(locationListenerNetwork);
		}
		public void onProviderDisabled(String provider) {}
		public void onProviderEnabled(String provider) {}
		public void onStatusChanged(String provider, int status, Bundle extras) {}
	};

	LocationListener locationListenerNetwork = new LocationListener() {
		public void onLocationChanged(Location location) {
			timer.cancel();
			locationResult.gotLocation(location);
			lm.removeUpdates(this);
			lm.removeUpdates(locationListenerGps);
		}
		public void onProviderDisabled(String provider) {}
		public void onProviderEnabled(String provider) {}
		public void onStatusChanged(String provider, int status, Bundle extras) {}
	};

	class GetLastLocation extends TimerTask {
		@Override
		public void run() {
			lm.removeUpdates(locationListenerGps);
			lm.removeUpdates(locationListenerNetwork);

			Location net_loc=null, gps_loc=null;
			if (gps_enabled) {
				gps_loc = lm.getLastKnownLocation(LocationManager.GPS_PROVIDER);
			}
			if (network_enabled) {
				net_loc = lm.getLastKnownLocation(LocationManager.NETWORK_PROVIDER);
			}

			//if there are both values use the latest one
			if (gps_loc!=null && net_loc!=null){
				if (gps_loc.getTime() > net_loc.getTime()) {
					locationResult.gotLocation(gps_loc);
				} else {
					locationResult.gotLocation(net_loc);
				}
				return;
			}

			if (gps_loc != null){
				locationResult.gotLocation(gps_loc);
				return;
			}
			if (net_loc != null){
				locationResult.gotLocation(net_loc);
				return;
			}
			locationResult.gotLocation(null);
		}
	}

	public static abstract class LocationResult{
		public abstract void gotLocation(Location location);
	}
}