package no.ntnu.item.arctis.examples.android.cityguide.androidloginui;

/* File: LoginActivity.java 
 * Author: Surya Kathayat
 * Description:	Activity which displays a simple login window
 * 				requesting username and password from the user. 
 */

import no.ntnu.item.arctis.android.R;
import no.ntnu.item.arctis.examples.android.cityguide.getperiodiclocationupdates.LocationService;
import no.ntnu.item.arctis.library.objects.login.Credentials;
import no.ntnu.item.arctis.library.proxies.actorrouter.android.ActorframeContainer;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.graphics.Color;
import android.os.Bundle;
import android.os.Handler;
import android.util.Log;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;



//implements OnClickListener to handle button clicks
public class LoginActivity extends Activity implements OnClickListener {

	private static String tag = "LoginGUI";

	private Button okButton;
	private Button cancelButton;
	private boolean loginInProgress;
	private Handler responseHandler;
	
	private static LoginActivity act;

	private TextView connSetting;
	
	private AndroidLoginUI androidLoginUI;

	@Override
	public void onCreate(Bundle savedInstanceState) {


		super.onCreate(savedInstanceState);

		setContentView(R.layout.login);

		//get buttons from xml layout
		this.okButton = (Button) findViewById(R.id.okButton);
		this.cancelButton = (Button) findViewById(R.id.cancelButton);

		act = this;

		okButton.setOnClickListener( this);
		cancelButton.setOnClickListener(this);

		this.loginInProgress = false;

		this.responseHandler = new Handler();

		// for the connection setting view to set the server addresses 
		connSetting = (TextView) findViewById(R.id.SettingsLabel);
		connSetting.setLinkTextColor(Color.MAGENTA );
		
		connSetting.setOnClickListener(new OnClickListener() {
			public void onClick(View v) {
				// doSomething();            		
				System.out.println("need to open a dialog for setting a server address");

				// show the alert for changing the server settings
				//ActorframeContainer.getContext().applicationSpecChanged("129.241.220.235");
				showServerConnectionSettingAlert();
			}
		});
		
		
	}


	/*
	 * A method to show a dialog that allows to change the server configuration
	 */
	private void showServerConnectionSettingAlert(){
		LayoutInflater infalter = LayoutInflater.from(this);  
		final View textEntryView = infalter.inflate(R.layout.server_conn_dialog, null);  
		AlertDialog.Builder dialogBuilder = new AlertDialog.Builder(this);  
		dialogBuilder.setTitle("Edit Server");  
		dialogBuilder.setView(textEntryView);  
		dialogBuilder.setPositiveButton("Save", new DialogInterface.OnClickListener(){ 
			public void onClick(DialogInterface dialog, int which)   
			{  				
				EditText mUserText = (EditText) textEntryView.findViewById(R.id.ALERT_DIALOG_EDIT_TEXT); 
				String newMsg = mUserText.getText().toString();

				// TODO send the changed spec to ActorRouter
				//ActorframeContainer.getContext().applicationSpecChanged(newMsg);

				// TODO update the userspec file
			}  

		});

		dialogBuilder.setNegativeButton("Cancel", new DialogInterface.OnClickListener(){ 
			public void onClick(DialogInterface dialog, int which)   
			{  				
				return;
			}  

		});


		dialogBuilder.show();
	}


	@Override
	public void onDestroy(){
		super.onDestroy();
	}

	@Override
	public void onResume(){
		super.onResume();
	}

	@Override
	//handle button clicks
	public void onClick(View v) {

		//check which of the two buttons is pressed
		boolean isOk =(v.getId() == R.id.okButton);
		Credentials cred = new Credentials(null,null);

		if(isOk){

			//do not process ok push while login in progress
			if(! this.loginInProgress){
				//fetch username and password for credentials
				EditText userNameField = (EditText)this.findViewById(R.id.UserNameField);
				EditText passWordField = (EditText)this.findViewById(R.id.PassWordField);

				cred = new Credentials(userNameField.getText().toString(),passWordField.getText().toString());

				//prevent sending login if username or password field is blank
				if((!cred.getUserName().equals("")) && (!cred.getPassword().equals(""))){
					setLoginStatus(true,"Press cancel to abort");
				
					AndroidLoginUI.getInstance().sendFromActivityToBlock("LOGIN", cred, act);
				}	

				else setLoginStatus(false, "Empty username/password field");
			}
		}
		//cancel button pressed
		else{
			if(this.loginInProgress){
				//send abort message to server for new login but do not close UI?
				Log.i(tag, "abort login");
				setLoginStatus(false,"Login aborted, enter username/password");
				//at.abortAnimation();
			}
			else{
				//no login in progress, close login UI
				//this.finish();
			}
		}
	}


	final Runnable loginOK = new Runnable() {
		public void run() {
			closeLoginUI();
		}
	};

	final Runnable loginNOK = new Runnable() {
		public void run() {
			notifyLoginFail();
		}
	};

	//Callback function for login
	public void loginGranted(){
		this.responseHandler.post(this.loginOK);
	}

	public void loginDenied(){
		this.responseHandler.post(this.loginNOK);
	}

	private void closeLoginUI(){
		finish();
		/*
		ActorframeContainer.loggedIn = true;
		Intent in = new Intent(this, MainActivity.class);
		this.startActivity(in);
		*/
		// Test --remove it
		// ActorframeContainer.pa_asm.sendMessage(new StartGameFromUser("Mamdu"), new ActorAddress("/configurationAgent","ConfigurationAgent"));

	}

	private void notifyLoginFail(){
		this.setLoginStatus(false, "Login denied, please try again");
	}

	//display status in the bottom part of the GUI
	//redefine (server messages ...)
	private void setLoginStatus(boolean b, String message){

		this.loginInProgress =b;
		if(b) ((TextView)this.findViewById(R.id.connectionStatusValue)).setText("Logging in"); 
		else  ((TextView)this.findViewById(R.id.connectionStatusValue)).setText("Offline"); 
		((TextView)this.findViewById(R.id.MessageValue)).setText(message);
	}
	
	
	 //exit if BACK KEY is pressed
	public boolean onKeyDown(int keyCode, KeyEvent event) {     
     Log.v("KEYCODE","" + keyCode);
		switch (keyCode) {
     case KeyEvent.KEYCODE_BACK:
     	Log.v("","Exit?");
		//AndroidLoginUI.getInstance().sendFromActivityToBlock("BACK");
     	finish();     	
     	
		ActorframeContainer.getContext().killActorFrameModel();
		ActorframeContainer.getContext().stopSelf();		 	 
		LocationService.getInstance().stopSelf();	
		
		android.os.Process.killProcess(android.os.Process.myPid());
     	break;
     } 
     return super.onKeyDown(KeyEvent.KEYCODE_0, event);
	}

}