package no.ntnu.item.arctis.library.android.location.getcurrentlocation;

import no.ntnu.item.arctis.runtime.AbstractRuntime;
import android.app.Service;
import android.content.Intent;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.os.IBinder;
import android.util.Log;

public class TheCurrentLocationService extends Service {

	private LocationManager locationManager;
	private final String TAG = "TheCurrentLocationService";
	public String provider = "gps";
	public String blockID;
	private String sessionID;
	protected boolean foundLocation = false;

	public TheCurrentLocationService() {
		// TODO Auto-generated constructor stub
	}

	public void onCreate() {
		Log.d(TAG, TAG + ":onCreate");
		super.onCreate();
	}

	public void onStart(Intent intent, int startId) {
		Log.d(TAG, TAG + ":onStart");
		// Actions to perform when service is started.
		blockID = intent.getStringExtra("BLOCK_ID");

		locationManager = (LocationManager) getSystemService(LOCATION_SERVICE);

		if (locationManager.isProviderEnabled(provider)) {
			requestLocUpdates(provider);
		} else {
			providerDisabled();
		}
		super.onStart(intent, startId);
	}

	public void onDestroy() {
		Log.d(TAG, TAG + ":onDestroy");
		if (locationlistener != null) {
			locationManager.removeUpdates(locationlistener);
		}
		if (!foundLocation) {
			sendSignalToArctis("onDestroy");
		}
		// stopSelf();
		super.onDestroy();
	}

	private void requestLocUpdates(String provider) {
		locationManager.requestLocationUpdates(provider, 0, 0, locationlistener);
	}

	LocationListener locationlistener = new LocationListener() {

		public void onLocationChanged(Location loc) {
			Log.d(TAG, TAG + ":onLocationChanged: " + loc.getLongitude());

			// This method sends Arctis signals
			broadcastLocationChangedSignal(loc);

			/* We found current location */
			foundLocation = true;
			
//			loc.get

			/*
			 * We are only interested in one location, i.e. the current location
			 * therefore we stop the service after first fix
			 */
			stopSelf();
		}

		public void onProviderDisabled(String arg0) {
			providerDisabled();
		}

		public void onProviderEnabled(String provider) {
		}

		public void onStatusChanged(String provider, int status, Bundle extras) {
		}
	};

	public IBinder onBind(Intent intent) {
		return null;
	}

	// Androids way of broadcasting signals is by use of intents
	// Arctis uses signals, so we should stick to signals and not use intents
	public void broadcastLocationChangedSignal(Location loc) {
		Log.d(TAG, TAG + ":broadcastLocationChangedSignal");
		sendSignalToArctisWithObject("LOCATION_CHANGED", loc);
	}

	private void providerDisabled() {
		sendSignalToArctis("PROVIDER_DISABLED");
		stopSelf();
	}

	private void sendSignalToArctis(String signalName) {
		AbstractRuntime.getRuntime(blockID).sendToBlock(sessionID,blockID, signalName + "_" + blockID);
	}

	private void sendSignalToArctisWithObject(String signalName, Object obj) {
		AbstractRuntime.getRuntime(blockID).sendToBlock(sessionID,blockID, signalName + "_" + blockID, obj);
	}

}
