package no.ntnu.item.arctis.library.android.location.locationprovider;

import no.ntnu.item.arctis.runtime.AbstractRuntime;
import android.app.Service;
import android.content.Intent;
import android.location.Criteria;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;
import android.util.Log;
import android.widget.Toast;

public class TheLocationService extends Service {

	private LocationManager locationManager;
	private final String TAG = "TheLocationService";
	public String provider;
	int time = 5000;
	int distance = 10;
	public Location cloc = null;
	public String blockID;
	private String sessionID;

	public TheLocationService() {
		// TODO Auto-generated constructor stub
	}

	public void onCreate() {
		super.onCreate();
		Log.d(TAG, TAG + ":onCreate");
	}

	public void onStart(Intent intent, int startId) {
		Log.d(TAG, TAG + ":onStart");
		// Actions to perform when service is started.
		time = intent.getIntExtra("time", 5000);
		distance = intent.getIntExtra("distance", 10);
		blockID = intent.getStringExtra("BLOCK_ID");
		Log.d(TAG, TAG + ":onStart->From intent; time= " + time + "distance= " + distance);
		if (cloc != null) {
			Log.d(TAG, TAG + ":onStart->coords now; lat: " + cloc.getLatitude() + " long: " + cloc.getLongitude());
		}

		locationManager = (LocationManager) getSystemService(LOCATION_SERVICE);

		provider = setCriterias();

		if (provider != null) {

			requestLocUpdates(provider);

			Location loc = locationManager.getLastKnownLocation(provider);
			Log.d(TAG, TAG + ":lastKnownLocation: " + loc.getLongitude());
		}

		else {
			noProviders();
		}

		// Tell Arctis that we have started, and send the handler reference
		sendSignalToArctisWithObject("onStart", arctisUpdateHandler);

		super.onStart(intent, startId);
	}

	public void onDestroy() {
		// TODO Auto-generated method stub
		Log.d(TAG, TAG + ":onDestroy");
		super.onDestroy();
		sendSignalToArctis("onResume");
	}

	private void requestLocUpdates(String provider) {
		locationManager.requestLocationUpdates(provider, (long) time, (float) distance, (LocationListener) new LocationListener() {

			public void onLocationChanged(Location loc) {
				Log.d(TAG, TAG + ":onLocationChanged: " + loc.getLongitude());
				cloc = loc;

				/*
				 * Toast.makeText(getBaseContext(), "Location changed : Lat: " +
				 * loc.getLatitude() + " Lng: " + loc.getLongitude(),
				 * Toast.LENGTH_SHORT).show();
				 */

				// This method uses intents
				// broadcastLocationChangedIntent(loc);

				// This method sends Arctis signals
				broadcastLocationChangedSignal(loc);
			}

			public void onProviderDisabled(String arg0) {
				providerDisabled();
			}

			public void onProviderEnabled(String provider) {
				providerEnabled();
			}

			public void onStatusChanged(String provider, int status, Bundle extras) {
				// TODO Auto-generated method stub

			}
		});

	}

	private String setCriterias() {

		Criteria criteria = new Criteria();
		criteria.setAccuracy(Criteria.ACCURACY_COARSE);
		criteria.setAltitudeRequired(false);
		criteria.setBearingRequired(false);
		criteria.setCostAllowed(false);
		criteria.setPowerRequirement(Criteria.NO_REQUIREMENT);

		String provider = locationManager.getBestProvider(criteria, true);

		Log.d(TAG, TAG + ":setCriteries return: " + provider);

		return provider;

	}

	@Override
	public IBinder onBind(Intent intent) {
		return null;
	}

	// Androids way of broadcasting signals is by use of intents
	// Arctis uses signals, so we should stick to signals and not use intents
	public void broadcastLocationChangedSignal(Location loc) {

		Log.d(TAG, TAG + ":broadcastLocationChangedSignal");
		sendSignalToArctisWithObject("LOCATION_CHANGED", loc);
	}

	public Location getLastKnownLocation() {
		Log.d(TAG, TAG + ":requestLastKnownLocation");
		Location loc = locationManager.getLastKnownLocation(provider);
		return loc;
	}

	private void noProviders() {
		Log.d(TAG, TAG + ":noProviders");
		Toast.makeText(getBaseContext(), "No location providers found. Enable gps or wifi, stopping service", Toast.LENGTH_SHORT).show();
		sendSignalToArctis("PROVIDER_DISABLED");
	}

	private void providerDisabled() {
		Log.d(TAG, TAG + ":providerDisabled");
		Toast.makeText(getBaseContext(), "No location providers found. Stopping Service", Toast.LENGTH_SHORT).show();
		sendSignalToArctis("PROVIDER_DISABLED");
	}

	private void providerEnabled() {
		Log.d(TAG, TAG + ":providerEnabled");
		Toast.makeText(getBaseContext(), "Location provider enabled again, updating every " + time + "msec and distance greater than " + distance, Toast.LENGTH_SHORT).show();
	}

	public Handler arctisUpdateHandler = new Handler() {

		public void handleMessage(Message msg) {
			switch (msg.what) {
			case 0:
				Toast.makeText(TheLocationService.this, msg.obj.toString(), Toast.LENGTH_SHORT).show();
				break;
			}
			super.handleMessage(msg);
		}

	};

	private void sendSignalToArctis(String signalName) {
		AbstractRuntime.getRuntime(blockID).sendToBlock(sessionID,blockID, signalName);

	}

	private void sendSignalToArctisWithObject(String signalName, Object obj) {
		AbstractRuntime.getRuntime(blockID).sendToBlock(sessionID,blockID, signalName, obj);

	}

}
