package no.ntnu.item.arctis.library.android.location.periodiclocationupdates;

import no.ntnu.item.arctis.runtime.AbstractRuntime;
import android.app.Service;
import android.content.Intent;
import android.location.Criteria;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.os.IBinder;
import android.util.Log;

public class TheLocationService extends Service {

	private LocationManager locationManager;
	private final String TAG = "TheLocationService";
	public String provider;
	int time = 60000;
	int distance = 20;
	public String blockID;
	private String sessionID;

	public TheLocationService() {
		// TODO Auto-generated constructor stub
	}

	public void onCreate() {
		super.onCreate();
	}

	public void onStart(Intent intent, int startId) {
		Log.d(TAG, TAG + ":onStart");
		// Actions to perform when service is started.
		time = intent.getIntExtra("time", 60000);
		distance = intent.getIntExtra("distance", 20);
		blockID = intent.getStringExtra("BLOCK_ID");

		locationManager = (LocationManager) getSystemService(LOCATION_SERVICE);

		provider = setCriterias();

		if (provider != null) {
			requestLocUpdates(provider);
		} else {
			noProviders();
		}

		// Tell Arctis that we have started
		sendSignalToArctis("onStart");

		super.onStart(intent, startId);
	}

	public void onDestroy() {
		Log.d(TAG, TAG + ":onDestroy");

		if (locationlistener != null) {
			locationManager.removeUpdates(locationlistener);
		}

		/* Tell Arctis we are killing the service */
		sendSignalToArctis("onDestroy");

		// stopSelf();
		super.onDestroy();
	}

	private void requestLocUpdates(String provider) {
		locationManager.requestLocationUpdates(provider, (long) time, (float) distance, locationlistener);
	}

	LocationListener locationlistener = new LocationListener() {

		public void onLocationChanged(Location loc) {
			Log.d(TAG, TAG + ":onLocationChanged");

			// This method sends Arctis signals
			broadcastLocationChangedSignal(loc);
		}

		public void onProviderDisabled(String arg0) {
			providerDisabled();
		}

		public void onProviderEnabled(String provider) {
			providerEnabled();
		}

		public void onStatusChanged(String provider, int status, Bundle extras) {
		}
	};

	private String setCriterias() {

		Criteria criteria = new Criteria();
		criteria.setAccuracy(Criteria.ACCURACY_FINE);
		criteria.setAltitudeRequired(false);
		criteria.setBearingRequired(false);
		criteria.setCostAllowed(false);
		// criteria.setPowerRequirement(Criteria.NO_REQUIREMENT);

		String provider = locationManager.getBestProvider(criteria, true);

		Log.d(TAG, TAG + ":setCriteries return: " + provider);

		return provider;

	}

	public IBinder onBind(Intent intent) {
		return null;
	}

	// Androids way of broadcasting signals is by use of intents
	// Arctis uses signals, so we should stick to signals and not use intents
	public void broadcastLocationChangedSignal(Location loc) {
		Log.d(TAG, TAG + ":broadcastLocationChangedSignal");
		sendSignalToArctisWithObject("LOCATION_CHANGED", loc);
	}

	private void noProviders() {
		Log.d(TAG, TAG + ":noProviders");
		sendSignalToArctis("NO_PROVIDERS");

	}

	private void providerDisabled() {
		Log.d(TAG, TAG + ":providerDisabled");
		sendSignalToArctis("PROVIDER_DISABLED");
	}

	private void providerEnabled() {
		Log.d(TAG, TAG + ":providerEnabled");
		sendSignalToArctis("PROVIDER_ENABLED");
	}

	private void sendSignalToArctis(String signalName) {
		AbstractRuntime.getRuntime(blockID).sendToBlock(sessionID,blockID, signalName + "_" + blockID);

	}

	private void sendSignalToArctisWithObject(String signalName, Object obj) {
		AbstractRuntime.getRuntime(blockID).sendToBlock(sessionID,blockID, signalName + "_" + blockID, obj);

	}

}
