package no.ntnu.item.arctis.library.android.alc;

import no.ntnu.item.arctis.android.R;
import no.ntnu.item.arctis.runtime.AbstractRuntime;
import no.ntnu.item.arctis.runtime.Block;
import no.ntnu.item.arctis.runtime.Logger;
import no.ntnu.item.arctis.runtime.android.AndroidBlock;
import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.Checkable;
import android.widget.TextView;

public class ArctisAndroidActivity extends Activity {
	
	private static String LOG_TAG = ArctisAndroidActivity.class.getSimpleName();

	public static final String BLOCK_ID_EXTRA_KEY = "BLOCK_ID";
	public static final String SESSION_ID_EXTRA_KEY = "SESSION_ID";
	public static final String LAYOUT_ID_EXTRA_KEY = "LAYOUT_ID";

	private String blockID;
	private String parentID;
	private String sessionID;

	private boolean createdPreviously = false;
	private boolean startedPreviously = false;
	
	public ArctisAndroidActivity() {
		Log.v(LOG_TAG, " +++++ observed CONSTRUCTOR " + this.getClass().getSimpleName() + " #" + this.hashCode());
	}
	
	protected void onNewIntent(Intent intent) {
		Log.v(LOG_TAG, " +++++ observed onNewIntent " + this.getClass().getSimpleName() + " #" + this.hashCode());
		String newBlockID = intent.getStringExtra(BLOCK_ID_EXTRA_KEY);
		String newSessionID = intent.getStringExtra(SESSION_ID_EXTRA_KEY);
		Log.v(LOG_TAG, "       task: " + getTaskId());
		Log.v(LOG_TAG, "       old blockID " + blockID);
		Log.v(LOG_TAG, "       new blockID " + newBlockID);
		
		Log.v(LOG_TAG, "       old sessionID " + sessionID);
		Log.v(LOG_TAG, "       new sessionID " + newSessionID);
		super.onNewIntent(intent);
	}



	public void onCreate(Bundle savedInstanceState) {
		if(AbstractRuntime.getRuntime()==null || (sessionID!=null && !AbstractRuntime.getRuntime().isSessionActive(sessionID))) {
			//Logger.getLogger().applicationError("onResume, but session " + sessionID + " seems inactive. Finishing activity " + this.getClass().getCanonicalName(), null);
			blockID = null;
			super.finish();
			return;
		}
		
		try {
			// setTheme(R.style.Theme_Dialog);
			super.onCreate(savedInstanceState);
			Log.v(LOG_TAG, " +++++ observed onCreate " + this.getClass().getSimpleName() + " #" + this.hashCode());
			Log.v(LOG_TAG, "       task: " + getTaskId());
			blockID = getIntent().getStringExtra(BLOCK_ID_EXTRA_KEY);
			sessionID = getIntent().getStringExtra(SESSION_ID_EXTRA_KEY);
			
			Log.v(LOG_TAG, "         blockID: " + blockID);
			Log.v(LOG_TAG, "       sessionID: " + sessionID);
			
			// check if runtime is executing, otherwise, start it
			if(blockID!=null && AbstractRuntime.getRuntime()!=null && AbstractRuntime.getRuntime().isRunning()) {
				if (getIntent().hasExtra(LAYOUT_ID_EXTRA_KEY)) {
					int layoutID = getIntent().getIntExtra(LAYOUT_ID_EXTRA_KEY, 0);
					setContentView(layoutID);
				}
				if (!createdPreviously) {
					Log.v(LOG_TAG, " +++++ sending onCreate #" + hashCode());
					AbstractRuntime.getRuntime().sendToBlock(sessionID, blockID, "onCreate_" + blockID, this);
					createdPreviously = true;
				} else {
					Log.v(LOG_TAG, " +++++ sending NOT onCreate #" + hashCode());
				}

			} else {
				blockID = null;
				Log.v(LOG_TAG, " +++++ ON create, detected terminated runtime system, starting service once we have finished this activity... " + hashCode());
			    setVisible(false);
			    setTitle("");
			    //setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);
//            Intent intent = new Intent(this, Start.class);
//            startService(intent);
			    finish();
			}
		} catch (Exception e) {
			Logger.getLogger().applicationError("Error in onCreate()", e);
		}
	}

	/**
	 * @deprecated 
	 * @param id
	 */
	public void setParentID(String id) {
		parentID = id;
	}

	public void onStart() {
		Log.v(LOG_TAG, " +++++ observed onStart " + this.getClass().getSimpleName() + " #" + this.hashCode());
		Log.v(LOG_TAG, "       task: " + getTaskId());
		
		String newBlockID = getIntent().getStringExtra(BLOCK_ID_EXTRA_KEY);
		String newSessionID = getIntent().getStringExtra(SESSION_ID_EXTRA_KEY);
		
		Log.v(LOG_TAG, "       old blockID " + blockID);
		Log.v(LOG_TAG, "       new blockID " + newBlockID);
		
		Log.v(LOG_TAG, "       old sessionID " + sessionID);
		Log.v(LOG_TAG, "       new sessionID " + newSessionID);
		
		try {
			super.onStart();
			if (!startedPreviously) {
				AndroidBlock.setCurrentActivityContext(this, blockID);
			}
			startedPreviously = true;
		} catch (Exception e) {
			Logger.getLogger().applicationError("Error in onResume()", e);
		}
	}

	public void onResume() {
		if(AbstractRuntime.getRuntime()==null || (sessionID!=null && !AbstractRuntime.getRuntime().isSessionActive(sessionID))) {
			//Logger.getLogger().applicationError("onResume, but session " + sessionID + " seems inactive. Finishing activity " + this.getClass().getCanonicalName(), null);
			blockID = null;
			super.finish();
		}
		
		
		Log.v(LOG_TAG, " +++++ observed onResume " + this.getClass().getSimpleName() + " #" + this.hashCode());
		Log.v(LOG_TAG, "       task: " + getTaskId());
		
		String newBlockID = getIntent().getStringExtra(BLOCK_ID_EXTRA_KEY);
		String newSessionID = getIntent().getStringExtra(SESSION_ID_EXTRA_KEY);
		
		Log.v(LOG_TAG, "       old blockID " + blockID);
		Log.v(LOG_TAG, "       new blockID " + newBlockID);
		
		Log.v(LOG_TAG, "       old sessionID " + sessionID);
		Log.v(LOG_TAG, "       new sessionID " + newSessionID);
		
		try {
			if(blockID==null || !AbstractRuntime.getRuntime().isRunning()) {
				blockID = null;
				Log.v(LOG_TAG, " +++++ ON resume, but with terminated runtime, so we finish this activity and start runtime service again.");
				finish();
			}
			super.onResume();
		} catch (Exception e) {
			Logger.getLogger().applicationError("Error in onResume()", e);
		}
	}

	public void onRestart() {
		super.onRestart();
		AndroidBlock.setCurrentActivityContext(this, blockID);
	}
	
	public void finish() {
		Log.v(LOG_TAG, " ----- observed finish() " + this.getClass().getSimpleName() + " #" + this.hashCode());
		Log.v(LOG_TAG, "       task: " + getTaskId());
		super.finish();
	}

	public void onDestroy() {
		Log.v(LOG_TAG, " ----- observed onDestroy " + this.getClass().getSimpleName() + " #" + this.hashCode());
		Log.v(LOG_TAG, "       task: " + getTaskId());
		try {
			super.onDestroy();
			if (/*isFinishing() && */blockID!=null) {
				Log.v(LOG_TAG, " ----- sending onDestroy " + hashCode());
				AbstractRuntime.getRuntime().sendToBlock(sessionID, blockID, "onDestroy_" + blockID);
			} else if (blockID==null) {
				Log.v(LOG_TAG, " ----- onDestroy, blockID is null, so we start the system again! " + hashCode());
			    
				Object startClass = AbstractRuntime.getRuntime().getContext("android-service-class");
				
				if(startClass!=null && startClass instanceof Class) {
					Intent intent = new Intent(this, (Class) startClass);
					intent.setAction("android.intent.action.MAIN");
					intent.addCategory("android.intent.category.LAUNCHER");
					startService(intent);
				} else {
					Logger.getLogger().applicationError("Could not get class to start the service. Is it added in the context?");
				}
			    
			} else {
				Log.v(LOG_TAG, " ----- sending NOT onDestroy, because isFinishing is false " + hashCode());
			}
		} catch (Exception e) {
			Logger.getLogger().applicationError("Error in onDestroy()", e);
		}
	}

	public void onBackPressed() {
		try {
			if(AbstractRuntime.getRuntime()==null || (sessionID!=null && !AbstractRuntime.getRuntime().isSessionActive(sessionID))) {
				Logger.getLogger().applicationError("onBack was pressed, but session " + sessionID + " seems inactive. Finishing activity " + this.getClass().getCanonicalName(), null);
				blockID = null;
				super.finish();
			}
			AbstractRuntime.getRuntime().sendToBlock(sessionID, blockID, "onBack_" + blockID);
		} catch (Exception e) {
			Logger.getLogger().applicationError("Error in onDestroy()", e);
		}
	}

	/**
	 * @deprecated
	 * @param signalName
	 * @param obj
	 */
	public void sendSignalToBuildingBlock(String signalName) {
		if(parentID!=null) {
			AbstractRuntime.getRuntime().sendToBlock(sessionID, parentID, signalName + "_" + parentID);
		} else {
			Logger.getLogger().applicationError("Parent ID is not set", null);
		}
	}

	/**
	 * @deprecated
	 * @param signalName
	 * @param obj
	 */
	public void sendSignalToBuildingBlockWithObject(String signalName, Object obj) {
		if(parentID!=null) {
			AbstractRuntime.getRuntime().sendToBlock(sessionID, parentID, signalName + "_" + parentID, obj);
		} else {
			Logger.getLogger().applicationError("Parent ID is not set", null);
		}
	}
	
	public void registerButton(int id, final String signal, final String blockID) {
		Button register = (Button) findViewById(id);
		register.setOnClickListener(new View.OnClickListener() {
			public void onClick(View v) {
				AbstractRuntime.getRuntime().sendToBlock(sessionID, blockID, signal + "_" + blockID);
			}
		});
	}
	
	public void registerButton(int id, final String signal, final String blockID, final Object data) {
		Button register = (Button) findViewById(id);
		register.setOnClickListener(new View.OnClickListener() {
			public void onClick(View v) {
				AbstractRuntime.getRuntime().sendToBlock(sessionID, blockID, signal + "_" + blockID, data);
			}
		});
	}
	
	public void setEnabled(int id, boolean enabled) {
		View view = findViewById(id);
		view.setEnabled(enabled);
	}
	
	public void setVisible(int id, boolean visible) {
		View view = findViewById(id);
		view.setVisibility(visible?View.VISIBLE:View.INVISIBLE);
	}
	
	public void setText(int id, String text) {
		TextView view = (TextView) findViewById(id);
		if(text!=null) {
			view.setText(text);
		} else {
			view.setText("");
		}
	}
	
	public boolean isChecked(int id) {
		return ((Checkable)findViewById(id)).isChecked();
	}
	
	public void setChecked(int id, boolean checked) {
		((Checkable)findViewById(id)).setChecked(checked);
	}
	
	
//	private String readString(int id) {
//		View view = activity.findViewById(id);
//		if(view instanceof TextView) {
//			return ((TextView)view).getText().toString();
//		} else {
//			return null;
//		}
//	}
//	
//	private boolean readBoolean(int id) {
//		View view = activity.findViewById(id);
//		if(view instanceof CheckBox) {
//			return ((CheckBox)view).isChecked();
//		} else {
//			return false;
//		}
//	}

}