package no.ntnu.item.arctis.library.android.loginui;

import no.ntnu.item.arctis.android.R;
import no.ntnu.item.arctis.library.objects.login.Credentials;
import no.ntnu.item.arctis.runtime.android.AndroidBlock;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;
import android.opengl.Visibility;
import android.preference.PreferenceManager;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.ViewFlipper;

public class LoginUI extends AndroidBlock {

	public static final String USERNAME_KEY = "username";
	public static final String PASSWORD_KEY = "password";

	public android.app.Activity activity;
	public no.ntnu.item.arctis.library.objects.login.Credentials credentials;

	private Button loginButton;
	private ViewFlipper viewFlipper;
	private EditText username;
	private EditText password;
	private TextView comment;
	private CheckBox savePassword;
	private TextView descriptionField;
	// Instance parameter. Edit only in overview page.
	public final java.lang.String description;
	// Instance parameter. Edit only in overview page.
	public final java.lang.String title;
	
	public Credentials getCredentials() {
		return new Credentials(username.getText().toString(), password.getText().toString());
	}

	public void registerListeners() {
		if(description != null && description.length() > 0){
			descriptionField = (TextView) activity.findViewById(R.id.descriptionField);
			descriptionField.setText(description);
			descriptionField.setVisibility(TextView.VISIBLE);
		}
		
		TextView titleView = (TextView) activity.findViewById(R.id.titleField);
		titleView.setText(title);
	
		loginButton = (Button) activity.findViewById(R.id.okButton);
		viewFlipper = (ViewFlipper) activity.findViewById(R.id.flipper);
		username = (EditText) activity.findViewById(R.id.loginDialogUsername);
		password = (EditText) activity.findViewById(R.id.loginDialogPassword);
		comment = (TextView) activity.findViewById(R.id.messageField);
		savePassword = (CheckBox) activity.findViewById(R.id.loginDialogSavePassword);
		loginButton.setOnClickListener(new OnClickListener() {
			public void onClick(View arg0) {
				storeCredentials();
				loginButton.setEnabled(false);
				username.setEnabled(false);
				password.setEnabled(false);
				viewFlipper.showNext();
				comment.setText("");
				sendToBlock("LOGIN");
			}
		});
	}

	protected void storeCredentials() {
		SharedPreferences prefs = PreferenceManager.getDefaultSharedPreferences(activity);
		Editor edit = prefs.edit();
		edit.putString(USERNAME_KEY, username.getText().toString());
		if (savePassword.isChecked()) {
			edit.putString(PASSWORD_KEY, password.getText().toString());
		} else {
			edit.remove(PASSWORD_KEY);
		}
		edit.commit();
	}

	public void displayReason(final String reason) {
		Runnable r = new Runnable() {
			public void run() {
				loginButton.setEnabled(true);
				username.setEnabled(true);
				password.setEnabled(true);
				comment.setText(reason != null ? reason : "");
				viewFlipper.showPrevious();
			}
		};
		getHandler().post(r);
	}

	public int getLayout() {
		return R.layout.login;
	}

	public Class getActivityClass() {
		return LoginUIActivity.class;
	}

	// Do not edit this constructor.
	public LoginUI(java.lang.String description, java.lang.String title) {
	    this.description = description;
	    this.title = title;
	}
}