package no.ntnu.item.arctis.library.android.notification;

import no.ntnu.item.arctis.runtime.AbstractRuntime;
import no.ntnu.item.arctis.runtime.android.AndroidBlock;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;

public class Notification extends AndroidBlock {

	private static final String INTENT_EXTRA_SIGNAL_ID = "SIGNAL_ID";
	private static final String INTENT_EXTRA_BLOCK_ID = "BLOCK_ID";

	private NotificationManager nm;
	private android.app.Notification notification;

	BroadcastReceiver br = new BroadcastReceiver() {
		public void onReceive(Context context, Intent intent) {
			String signalID = intent.getStringExtra(INTENT_EXTRA_SIGNAL_ID);
			String blockID = intent.getStringExtra(INTENT_EXTRA_BLOCK_ID);
			AbstractRuntime.getRuntime(blockID).sendToBlock(sessionID,blockID, signalID);
			context.unregisterReceiver(this);
		}
	};

	// Parameter defined by UML. Do not edit.
	public final java.lang.String title;
	// Parameter defined by UML. Do not edit.
	public final boolean sound;

	public void displayNotification(final String text) {
		final Context context = getContext();

		Runnable run = new Runnable() {
			public void run() {
				nm = (NotificationManager) context.getSystemService(Context.NOTIFICATION_SERVICE);

				// lag intent
				Intent intent = new Intent(blockID);
				intent.putExtra(INTENT_EXTRA_SIGNAL_ID, "NOTIFICATION_PRESSED" + "_" + blockID);
				intent.putExtra(INTENT_EXTRA_BLOCK_ID, blockID);
				PendingIntent contentIntent = PendingIntent.getBroadcast(context, 0, intent, 0);

				// lag delete intent
				Intent d = new Intent(blockID);
				d.putExtra(INTENT_EXTRA_SIGNAL_ID, "NOTIFICATION_CLEARED" + "_" + blockID);
				d.putExtra(INTENT_EXTRA_BLOCK_ID, blockID);
				PendingIntent deleteIntent = PendingIntent.getBroadcast(context, 0, d, 0);

				// lag notification
				notification = new android.app.Notification(0x7f020001, title, System.currentTimeMillis());
				notification.setLatestEventInfo(context, title, text, contentIntent);
				notification.deleteIntent = deleteIntent;
				notification.flags |= android.app.Notification.FLAG_AUTO_CANCEL;
				if (sound) {
					notification.defaults |= android.app.Notification.DEFAULT_SOUND;
				}
				// we use the hash code of the blockID as identifier for the
				// notification
				nm.notify(blockID.hashCode(), notification);

			}
		};
		context.registerReceiver(br, new IntentFilter(blockID));
		getHandler().post(run);
	}

	public void deleteNotification() {
		final Context context = getContext();
		Runnable run = new Runnable() {
			public void run() {
				nm = (NotificationManager) context.getSystemService(Context.NOTIFICATION_SERVICE);
				nm.cancel(blockID.hashCode());
			}
		};
		context.registerReceiver(br, new IntentFilter(blockID));
		getHandler().post(run);
	}

	public void updateNotification(final String title) {
		final Context context = getContext();

		Runnable run = new Runnable() {
			public void run() {
				nm = (NotificationManager) context.getSystemService(Context.NOTIFICATION_SERVICE);

				// create intent
				Intent intent = new Intent(blockID);
				intent.putExtra(INTENT_EXTRA_SIGNAL_ID, "NOTIFICATION_PRESSED" + "_" + blockID);
				intent.putExtra(INTENT_EXTRA_BLOCK_ID, blockID);
				PendingIntent contentIntent = PendingIntent.getBroadcast(context, 0, intent, 0);

				// create deletion intent
				Intent d = new Intent(blockID);
				d.putExtra(INTENT_EXTRA_SIGNAL_ID, "NOTIFICATION_CLEARED" + "_" + blockID);
				d.putExtra(INTENT_EXTRA_BLOCK_ID, blockID);
				PendingIntent deleteIntent = PendingIntent.getBroadcast(context, 0, d, 0);

				notification.setLatestEventInfo(context, title, "new text", contentIntent);
				notification.deleteIntent = deleteIntent;
				notification.flags |= android.app.Notification.FLAG_AUTO_CANCEL;
				if (sound) {
					notification.defaults |= android.app.Notification.DEFAULT_SOUND;
				}
				nm.notify(blockID.hashCode(), notification);
			}
		};
		context.registerReceiver(br, new IntentFilter(blockID));
		getHandler().post(run);
	}

	// Do not edit this constructor.
	public Notification(java.lang.String title, boolean sound) {
		this.title = title;
		this.sound = sound;
	}
}